from numpy import pi

from ..domain    import Domain
from ..field     import Field
from ..fieldlist import FieldList
from ..units     import Units

from ..data.data import Data

_d360    = Data(360.0, 'degrees_east')
_metres2 = Units('m2')

>>> cf.weights(f, 'time: linear height: mass: XY: area')
>>> cf.weights(f, {'time': 'linear', 'height': 'mass', 'YX': 'area'})

>>> cf.weights(f, 'time: equal height: mass: XY: area')
>>> cf.weights(f, 'height: mass: lat,lon: area')
>>> cf.weights(f, {'time': 'equal', 'height': 'mass', 'XY': 'area'})
>>> cf.weights(f, {'time': None, 'height': 'mass', 'lat,lon': 'area'})

>>> cf.weights(f, 'height: mass: lat,lon: area', normalize=False)
>>> cf.weights(f, 'height: mass: lat,lon: area', normalize=False, reduce=False)

>>> cf.weights(f, 'Z: mass: XY: area', normalize=False, reduce=False)

>>> cf.weights(f, 'ZXY: volume', normalize=False, reduce=False)
>>> cf.weights(f, 'height,lat,lon: volume', normalize=False, reduce=False)
>>> cf.weights(f, 'sigma,lat,lon: volume', normalize=False, reduce=False)

def weights(f, w, normalize=True, reduce=True, r=6371229.0):


    out = FieldList()

    in_field = isinstance(f, Field)

    for g in flat(f):
        oof = {}
        for key, value in arse.iteritems():
            d = len(key)
            if re_search('[TXYZ]', key):
                if len(key) == 1:
                    item = self.coord(ctype=key, rank=1)
                    oof[key] = item
                else:
                    # check for commas
                    for k in key:
                        items = self.coords(ctype=k, rank=1)
                        if len(items) == 1:
                            K, item = items.popitem()
                            axes.update(self.axes(K))
                        oof[k] = item
            else:
                for k in key.split(','):
                    item = self.coord(k, rank=1)
                    ctype = item.ctype
                    if ctype:
                        k = ctype
                    else:
                        k = item.identity()
                    oof[k] = item

        
            # check for cm
            if d == 2 and value == 'area':
                key_cm, cm = self.cm('area', axes=axes, equalset=True)
                oof['XY'] = cm
            elif d == 3 and value == 'volume':
                key_cm, cm = self.cm('volume', axes=axes, equalset=True)
                oof[(X, None, Y)] = cm
                
        types = arse.values)(
            
            if set('XY').issubset(key):
                # ------------------------------------------------------------
                # Find 1-d (grid) latitude and longitude coordinates
                # ------------------------------------------------------------
                lat = self.coords(ctype='Y', rank=1)
                lon = self.coords(ctype='X', rank=1)
                if not (len(lon) == 1 and len(lat) == 1):
                    lat = self.coords('grid_latitude', rank=1)
                    lon = self.coords('grid_longitude', rank=1)
                    if not (len(lon) == 1 and len(lat) == 1):
                        raise ValueError("Can't find 1-d lat/lon coordinate objects")
            #--- End: if
            if set('Z').issubset(key):
                # ------------------------------------------------------------
                # Find 1-d (grid) latitude and longitude coordinates
                # ------------------------------------------------------------
                Z = self.coord(ctype='Z', rank=1)
                if Z is None:
                    raise ValueError("Can't find 1-d Z coordinate object")
               
            if set('XYZ').issubset(key):
                # Look for volume cell measure
                key_cm, cm = self.cms('volume', axes=axes, strict_axes=True)
                if len(cm) != 1:
                    data = cm.data.copy()
                    

        # ------------------------------------------------------------
        # Find 1-d (grid) latitude and longitude coordinates
        # ------------------------------------------------------------
        lat = self.coords(ctype='Y', rank=1)
        lon = self.coords(ctype='X', rank=1)
        if not (len(lon) == 1 and len(lat) == 1):
            lat = self.coords('grid_latitude', rank=1)
            lon = self.coords('grid_longitude', rank=1)
            if not (len(lon) == 1 and len(lat) == 1):
                raise ValueError("Can't find 1-d lat/lon coordinate objects")
        #--- End: if
        
        key_lat, lat = lat.popitem()
        key_lon, lon = lon.popitem()

        # ------------------------------------------------------------
        # Create a domain for the output field of cell areas
        # ------------------------------------------------------------
        axes = f.axes((key_lat, key_lon))

        domain_kwargs = {'copy': False}
        if len(axes) == 1:
            # The (grid) latitude and longitude coordinates are
            # auxiliary to a common axis. See example 7.3 in the CF
            # conventions 1.6.
            domain_kwargs['aux'] = (lat.copy(), lon.copy())
            new_key_lat, new_key_lon = 'aux0', 'aux1'
        else:
            # The (grid) latitude and longitude coordinates are
            # dimension coordinates
            domain_kwargs['dim'] = (lat.copy(), lon.copy())
            new_key_lat, new_key_lon = 'dim0', 'dim1'
        #--- End: if

        # Find the any grid mapping transforms for the (grid) latitude
        # and longitude coordinates
        trans = []
        earth_radius = None
        for transform in self.transforms().itervalues():
            if not transform.isgrid_mapping:
                continue

            if 'earth_radius' in transform:
                if earth_radius is None:
                    earth_radius = transform['earth_radius']
                else:
                    raise ValueError("Multiple earth radii!")
            #--- End: if
                    
            if len(transform.coordinates) == 2:
                transform = transform.copy()
                try:
                    transform.rename_pointer(key_lat, new_key_lat)
                    transform.rename_pointer(key_lon, new_key_lon)
                except ValueError:
                    pass
                else:
                    trans.append(transform)
        #--- End: for
        if trans:                
            domain_kwargs['trans'] = trans

        if earth_radius is None:
            # Use default earth radius in metres
            earth_radius = r
             
        # ------------------------------------------------------------
        # Try to find a cell measure for the (grid) latitude and
        # longitude axes
        # ------------------------------------------------------------
        key_cm, cm = self.cms('area', axes=axes, strict_axes=True)
        if len(cm) != 1:
            data = cm.data.copy()

            # Check for transposed cell meausure data array
            if (cm.ndim == 2 and 
                (self.axes(key_cm, ordered=True)[0] != 
                 self.axes(key_lat, ordered=True)[0])):
                 domain_kwargs['assign_axes'] = {'data': ['dim1', 'dim0']}
            #--- End: if

         else:
            
             # ------------------------------------------------------------
             # Calculate the area of each horizontal grid cell
             # ------------------------------------------------------------
             # Calculate the area of each globally wrapping zonal band
             bounds = lat.get_bounds(create=infer_bounds).data
             bounds.clip(-90, 90, 'degrees_north')
             bounds.sin()
             zonal_area = ((bounds[:,1]-bounds[:,0])*2*pi*earth_radius**2).abs()
             
             # Calculate the fraction of a globally wrapping zonal band
             # occupied by each meridional segment
             bounds = lon.get_bounds(create=infer_bounds).data
             meridional_fraction  = ((bounds[:, 1] - bounds[:, 0])/_d360).abs()
             meridional_fraction.transpose()
             
             data = zonal_area*meridional_fraction
             data.override_units(_metres2)
        #--- End: if         
            
        # Create an output field with the horizontal cell areas
        out.append(Field(properties={'standard_name': 'area'},
                         domain=Domain(**domain_kwargs),
                         data=data,
                         data_axes=data_axes,
                         copy=False))
        
       #try:
        #    indices = x.indices(key_lat=cf_set((-90, 90), 'degrees_north'))
        #except IndexError:
        #    pass
        #else:
        #    condition = x.mask
        #    condition[...] = cf_masked
        #    condition[indices] = True
        #    x.Data.setdata(x.Data[indices]*meridional_fraction, None, condition)
        ##--- End: try
    #--- End: for

    if field_in:
        return  out[0]
    else:
        return out
