import cf
import numpy
import os
import itertools

def test(chunk_sizes=(17, 34, 60, 300, 1000000)):
    print '----------------------------------------------------------'
    print 'cf.Data'
    print '----------------------------------------------------------'
    original_chunksize = cf.CHUNKSIZE()

    for chunksize in chunk_sizes[::-1]:

        cf.CHUNKSIZE(chunksize)

        # ----------------------------------------------------------------
        # cf.Data.asdatetime
        # cf.Data.asreftime
        # ----------------------------------------------------------------
        d = cf.Data([[1.93, 5.17]], 'days since 2000-12-29')
        assert(d.dtype == numpy.dtype(float))
        assert(d._isdt == False)
        d.asreftime()
        assert(d.dtype == numpy.dtype(float))
        assert(d._isdt == False)
        d.asdatetime()
        assert(d.dtype == numpy.dtype(object))
        assert(d._isdt == True)
        d.asdatetime()
        assert(d.dtype == numpy.dtype(object))
        assert(d._isdt == True)
        d.asreftime()
        assert(d.dtype == numpy.dtype(float))
        assert(d._isdt == False)
        print "cf.Data.asdatetime passed", "pmshape =", d.pmshape
        print "cf.Data.asreftime passed", "pmshape =", d.pmshape
        
        # --------------------------------------------------------------------
        # cf.Data.datum
        # --------------------------------------------------------------------
        d = cf.Data(5, 'metre')
        assert(d.datum() == 5)
        assert(d.datum(0) == 5)
        assert(d.datum(-1) == 5)
        
        for d in [cf.Data([4, 5, 6, 1, 2, 3], 'metre'),
                  cf.Data([[4, 5, 6], [1, 2, 3]], 'metre')]:
            assert(d.datum(0) == 4)
            assert(d.datum(-1) == 3)
            for index in d.ndindex():
                assert(d.datum(index)  == d.array[index].item())
                assert(d.datum(*index) == d.array[index].item())
        #--- End: for
        
        d = cf.Data(5, 'metre')
        d[()] = cf.masked
        assert(d.datum() is cf.masked)
        assert(d.datum(0) is cf.masked)
        assert(d.datum(-1) is cf.masked)
        print "cf.Data.datum passed", "pmshape =", d.pmshape
        
        # ----------------------------------------------------------------
        # cf.Data.min
        # ----------------------------------------------------------------
        d = cf.Data([[4, 5, 6], [1, 2, 3]], 'metre')
        assert(d.min == cf.Data(1, 'metre'))
        assert(d.min.datum() == 1)
        d[1, 0] = cf.masked
        assert(d.min == 2)
        assert(d.min.datum() == 2)
        assert(d.min == cf.Data(0.002, 'km'))
        print "cf.Data.min passed", "pmshape =", d.pmshape
        
        # ----------------------------------------------------------------
        # cf.Data.max
        # ----------------------------------------------------------------
        d = cf.Data([[4, 5, 6], [1, 2, 3]], 'metre')
        assert(d.max == cf.Data(6, 'metre'))
        assert(d.max.datum() == 6)
        d[0, 2] = cf.masked
        assert(d.max == 5)
        assert(d.max.datum() == 5)
        assert(d.max == cf.Data(0.005, 'km'))
        print "cf.Data.max passed", "pmshape =", d.pmshape
        
        # --------------------------------------------------------------------
        # cf.Data.ndindex
        # --------------------------------------------------------------------
        for d, shape in ([cf.Data(5, 'metre'), ()],
                         [cf.Data([4, 5, 6, 1, 2, 3], 'metre'), (6,)],
                         [cf.Data([[4, 5, 6], [1, 2, 3]], 'metre'), (2, 3)]):
            for i, j in zip(d.ndindex(), numpy.ndindex(shape)):
                assert(i == j)
        #--- End: for
        print "cf.Data.ndindex passed", "pmshape =", d.pmshape

#        # ----------------------------------------------------------------
#        # cf.Data.rollaxis
#        # ----------------------------------------------------------------
#        a = numpy.arange(10*15*19).reshape(10, 1, 15, 19)
#        d = cf.Data(a.copy())
#
#        indices = range(a.ndim) + range(-a.ndim, 0)
#        
#        for key in (False, True):
#            for axis in indices:
#                for p in indices:
#                    print '\na.shape, d.shape=',a.shape, d.shape, axis, p
#                    a = numpy.rollaxis(a, axis, p)
#                    if not key:
#                        d.rollaxis(axis, p)
#                    else:
#                        d.rollaxis(d._dimensions[axis], p)
#                    print '  a.shape, d.shape=',a.shape, d.shape, axis, p
#
#                    message = 'd.shape='+repr(d.shape)+ ', a.shape='+repr(a.shape)
#                    assert d.shape == a.shape, message
#                    assert (d.array == a).all(), message
#                #--- End: for
#            #--- End: for
#        #--- End: for

        print "cf.Data.rollaxis passed", "pmshape =", d.pmshape
        
        # ----------------------------------------------------------------
        # cf.Data.transpose
        # ----------------------------------------------------------------
        a = numpy.arange(10*15*19).reshape(10, 1, 15, 19)
        d = cf.Data(a.copy())

        indices = range(a.ndim) # + range(-a.ndim, 0)
        
        for key in (False, True):
            for indices in (range(a.ndim), range(-a.ndim, 0)):
                for axes in itertools.permutations(indices):
                    a = numpy.transpose(a, axes)
                    if not key:
                        d.transpose(axes)
                    else:
                        axes = list(axes)
                        axes[-2] = d._dimensions[axes[-2]]
                        d.transpose(axes)

                    message = 'd.shape='+repr(d.shape)+ ',a.shape='+repr(a.shape)
                    assert d.shape == a.shape, message
                    assert (d.array == a).all(), message
                #--- End: for
            #--- End: for
        #--- End: for

        print "cf.Data.transpose passed", "pmshape =", d.pmshape
        
        # ----------------------------------------------------------------
        # cf.Data.unique
        # ----------------------------------------------------------------
        d = cf.Data([[4, 2, 1], [1, 2, 3]], 'metre')
        assert((d.unique == cf.Data([1, 2, 3, 4], 'metre')).all())
        d[1, -1] = cf.masked
        assert((d.unique == cf.Data([1, 2, 4], 'metre')).all())
        print "cf.Data.unique passed", "pmshape =", d.pmshape
        
        # ----------------------------------------------------------------
        # Broadcasting
        # ----------------------------------------------------------------
        A = [numpy.array(3),
             numpy.array( [3]),
             numpy.array( [3]).reshape(1, 1),
             numpy.array( [3]).reshape(1, 1, 1),
             numpy.arange(  5).reshape(5, 1),
             numpy.arange(  5).reshape(1, 5),
             numpy.arange(  5).reshape(1, 5, 1),
             numpy.arange(  5).reshape(5, 1, 1),
             numpy.arange(  5).reshape(1, 1, 5),
             numpy.arange( 25).reshape(1, 5, 5),
             numpy.arange( 25).reshape(5, 1, 5),
             numpy.arange( 25).reshape(5, 5, 1),
             numpy.arange(125).reshape(5, 5, 5),
             ]
        
        for a in A:
            for b in A:
                d = cf.Data(a)
                e = cf.Data(b)
                ab = a*b
                de = d*e

                message = 'de.shape='+repr(de.shape)+ ',ab.shape='+repr(ab.shape)
                assert de.shape == ab.shape, message
                assert (de.array == ab).all(), message
            #--- End: for
        #--- End: for
        print "cf.Data broadcasting passed"
        
        # ----------------------------------------------------------------
        # __contains__
        # ----------------------------------------------------------------
        d = cf.Data([[0.0, 1,  2], [3, 4, 5]], units='m')
        x = 4 in d
        x &= cf.Data(3) in d
        x &= cf.Data([[[[3]]]]) in d
        value = d[1, 2]
        value.Units *= 2
        value.squeeze(0)
        x &= value in d
        x &= numpy.array([[[2]]]) in d
        assert(x)
        print "cf.Data.__contains__ passed", "pmshape =", d.pmshape
        
        # ----------------------------------------------------------------
        # cf.Data.year
        # cf.Data.month
        # cf.Data.day
        # cf.Data.hour
        # cf.Data.minute
        # cf.Data.second
        # ----------------------------------------------------------------
        d = cf.Data([[1.93, 5.17]], 'days since 2000-12-29')
        assert(d.year.equals(cf.Data([[2000, 2001]])))
        assert(d.month.equals(cf.Data([[12, 1]])))
        assert(d.day.equals(cf.Data([[30, 3]])))
        assert(d.hour.equals(cf.Data([[22, 4]])))
        assert(d.minute.equals(cf.Data([[19, 4]])))
        assert(d.second.equals(cf.Data([[12, 48]])))
        
        d = cf.Data([[1.93, 5.17]], cf.Units('days since 2000-12-29', '360_day'))
        assert(d.year.equals(cf.Data([[2000, 2001]])))
        assert(d.month.equals(cf.Data([[12, 1]])))
        assert(d.day.equals(cf.Data([[30, 4]])))
        assert(d.hour.equals(cf.Data([[22, 4]])))
        assert(d.minute.equals(cf.Data([[19, 4]])))
        assert(d.second.equals(cf.Data([[12, 48]])))
        
        print 'cf.Data.year passed', "pmshape =", d.pmshape
        print 'cf.Data.month passed', "pmshape =", d.pmshape
        print 'cf.Data.day passed', "pmshape =", d.pmshape
        print 'cf.Data.hour passed', "pmshape =", d.pmshape
        print 'cf.Data.minute passed', "pmshape =", d.pmshape
        print 'cf.Data.second passed', "pmshape =", d.pmshape
        
        # ----------------------------------------------------------------
        # Binary and unary operators
        # ----------------------------------------------------------------
        array = numpy.array([[1.93, 5.17]])
        d = cf.Data(array, 'metre')
        
        x = 2
        assert((d + x).equals(cf.Data(array + x, 'm'), traceback=1))
        assert((d * x).equals(cf.Data(array * x, 'm'), traceback=1))
        assert((d / x).equals(cf.Data(array / x, 'm'), traceback=1))
        assert((d - x).equals(cf.Data(array - x, 'm'), traceback=1))
        assert((d // x).equals(cf.Data(array // x, 'm'), traceback=1))
        assert((d ** x).equals(cf.Data(array ** x, 'm2'), traceback=1))
        assert(d.__truediv__(x).equals(cf.Data(array.__truediv__(x), 'm'), traceback=1))
        print 'cf.Data + %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data * %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data / %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data - %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data // %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data ** %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data.__truediv__(%s) passed' % repr(x), "pmshape =", d.pmshape
        
        assert((x + d).equals(cf.Data(x + array, 'm'), traceback=1))
        assert((x * d).equals(cf.Data(x * array, 'm'), traceback=1))
        assert((x / d).equals(cf.Data(x / array, 'm-1'), traceback=1))
        assert((x - d).equals(cf.Data(x - array, 'm'), traceback=1))
        assert((x // d).equals(cf.Data(x // array, 'm-1'), traceback=1))
        assert((x ** d).equals(cf.Data(x ** array, 'm2'), traceback=1))
        assert(d.__rtruediv__(x).equals(cf.Data(array.__rtruediv__(x), 'm-1'), traceback=1))
        print 'cf.Data %s + passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data %s * passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data %s / passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data %s - passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data %s // passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data %s ** passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data.__rtruediv__(%s) passed' % repr(x), "pmshape =", d.pmshape
        
        e = d.copy()
        a = array.copy()
        e += 2
        assert(e.equals(cf.Data(a + 2, 'm')))
        e = d.copy()
        a = array.copy()
        e *= 2
        assert(e.equals(cf.Data(a * 2, 'm')))
        e = d.copy()
        a = array.copy()
        e /= 2
        assert(e.equals(cf.Data(a / 2, 'm')))
        e = d.copy()
        a = array.copy()
        e -= 2
        assert(e.equals(cf.Data(a - 2, 'm')))
        e = d.copy()
        a = array.copy()
        e //= 2
        assert(e.equals(cf.Data(a // 2, 'metre')))
        e = d.copy()
        a = array.copy()
        e **= 2
        assert(e.equals(cf.Data(a ** 2, 'm2')))
        e = d.copy()
        a = array.copy()
        e.__itruediv__(2)
        assert(e.equals(cf.Data(a.__itruediv__(2), 'm')))
        
        print 'cf.Data += %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data *= %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data /= %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data -= %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data //= %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data **= %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data.__itruediv__(%s) passed' % repr(x), "pmshape =", d.pmshape
        
        x = cf.Data(2, 'm')
        assert((d + x).equals(cf.Data(array + x.datum(), 'm'), traceback=1))
        assert((d * x).equals(cf.Data(array * x.datum(), 'm2'), traceback=1))
        assert((d / x).equals(cf.Data(array / x.datum(), ''), traceback=1))
        assert((d - x).equals(cf.Data(array - x.datum(), 'm'), traceback=1))
        assert((d // x).equals(cf.Data(array // x.datum(), ''), traceback=1))
        try:
            d ** x
        except ValueError:
            print 'cf.Data ** %s passed' % repr(x), "pmshape =", d.pmshape
        assert(d.__truediv__(x).equals(cf.Data(array.__truediv__(x.datum()), ''), traceback=1))
        print 'cf.Data + %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data * %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data / %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data - %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data // %s passed' % repr(x), "pmshape =", d.pmshape
        print 'cf.Data.__truediv__(%s) passed' % repr(x), "pmshape =", d.pmshape
        
        
        # ----------------------------------------------------------------
        # cf.Data.all
        # cf.Data.any
        # ----------------------------------------------------------------
        d = cf.Data([[0] * 1000])
        assert(not d.any())
        assert(not d.all())
        
        d[-1,-1] = 1
        assert(d.any())
        assert(not d.all())
        
        d[...] = 1
        assert(d.any())
        assert(d.all())
        print 'cf.Data.all passed', "pmshape =", d.pmshape
        print 'cf.Data.any passed', "pmshape =", d.pmshape

        # ----------------------------------------------------------------
        # cf.Data.__setitem__
        # ----------------------------------------------------------------
        for hardmask in (False, True):
            a = numpy.ma.arange(3000).reshape(50, 60)
            if hardmask:
                a.harden_mask()
            else:
                a.soften_mask()
                
            d = cf.Data(a.filled(), 'm')
            d.hardmask = hardmask

            for n, (j, i) in enumerate(((34, 23), (0, 0), (-1, -1),
                                        (slice(40, 50), slice(58, 60)),
                                        (Ellipsis, Ellipsis),
                                        )):
                n = -n-1
                for dvalue, avalue in ((n, n), (cf.masked, numpy.ma.masked), (n, n)):
                    message = "Failed in __setitem__(%s, %s)=%s" % (j, i, dvalue)
                    d[j, i] = dvalue
                    a[j, i] = avalue
                    assert (d.array == a).all() in (True, numpy.ma.masked), message
                    assert (d.mask.array == numpy.ma.getmaskarray(a)).all(), message
            #--- End: for

            a = numpy.ma.arange(3000).reshape(50, 60)
            if hardmask:
                a.harden_mask()
            else:
                a.soften_mask()

            d = cf.Data(a.filled(), 'm')
            d.hardmask = hardmask

            (j, i) = (slice(0, 2), slice(0, 3))
            array = numpy.array([[1, 2, 6],[3, 4, 5]])*-1
            for dvalue in (array,
                           numpy.ma.masked_where(array < -2, array),
                           array):
                message = "Failed in __setitem__(%s, %s)=%s" % (j, i, dvalue)
                d[j, i] = dvalue
                a[j, i] = dvalue
                assert (d.array == a).all() in (True, numpy.ma.masked), message
                assert (d.mask.array == numpy.ma.getmaskarray(a)).all(), message
            #--- End: for

            print 'cf.Data.__setitem__ passed: hardmask =',hardmask,', pmshape =', d.pmshape
        #--- End: for


        print ''
        print 'cf.Data with chunk size %d passed' % chunksize
        print ''
    #--- End: for

    cf.CHUNKSIZE(original_chunksize)

    print
    print '---------------------------------------------------------------------------'
    print 'All cf.Data tests passed for cf version', cf.__version__
    print 'Running from', os.path.abspath(cf.__file__)
    print '---------------------------------------------------------------------------'  
    print
#--- End: def

if __name__ == "__main__":
    test()
